#include "Logger.h"
#include <iostream>
#include <chrono>
#include <ctime>
#include <cstdarg>
namespace OX
{
    std::vector<Message> Logger::m_messages;

    inline const char* ToString(MessageType type) {
        switch (type) {
            case MessageType::Info:    return "Info";
            case MessageType::Warning: return "Warning";
            case MessageType::Error:   return "Error";
            case MessageType::Debug:   return "Debug";
            case MessageType::Ok:      return "Ok";
            default:                   return "Unknown";
        }
    }



    static std::string GetTimestamp() {
        auto now = std::chrono::system_clock::now();
        std::time_t t = std::chrono::system_clock::to_time_t(now);
        std::tm* tm = std::localtime(&t);
        char buffer[20];
        std::strftime(buffer, sizeof(buffer), "%H:%M:%S", tm);
        return buffer;
    }

    static const char* GetAnsiColor(MessageType type) {
        switch (type) {
            case MessageType::Info:    return "\033[1;34m";
            case MessageType::Warning: return "\033[1;33m";
            case MessageType::Error:   return "\033[1;31m";
            case MessageType::Debug:   return "\033[1;90m";
            case MessageType::Ok:      return "\033[1;32m";
            default:                   return "\033[0m";
        }
    }


    static void GetRGBColor(MessageType type, float& r, float& g, float& b) {
        switch (type) {
            case MessageType::Info:    r = 0.2f; g = 0.6f; b = 1.0f; break;
            case MessageType::Warning: r = 1.0f; g = 0.8f; b = 0.2f; break;
            case MessageType::Error:   r = 1.0f; g = 0.2f; b = 0.2f; break;
            case MessageType::Debug:   r = 0.7f; g = 0.7f; b = 0.7f; break;
            case MessageType::Ok:      r = 0.6f; g = 1.0f; b = 0.6f; break;
            default:                   r = g = b = 1.0f; break;
        }
    }


    void Logger::LogInternal(const std::string& text, MessageType type) {
        float r, g, b;
        GetRGBColor(type, r, g, b);

        Message message {
            GetTimestamp(),
            text,
            type,
            r, g, b
        };

        m_messages.push_back(message);

        std::cout   << "\033[37m[\033[35mONYX\033[37m]"
                    << "\033[37m[\033[36m" << message.timestamp
                    << "\033[37m]\033[37m[\033[33m"
                    << GetAnsiColor(type)
                    << ToString(message.type)
                    << "\033[37m] "
                    << GetAnsiColor(type)
                    << text
                    << "\033[0m\n";

    }

    void Logger::Logf(MessageType type, const char* format, ...) {
        char buffer[1024];

        va_list args;
        va_start(args, format);
        std::vsnprintf(buffer, sizeof(buffer), format, args);
        va_end(args);

        LogInternal(std::string(buffer), type);
    }

    void Logger::LogInfo(const char* format, ...) {
        va_list args;
        va_start(args, format);
        char buffer[1024];
        std::vsnprintf(buffer, sizeof(buffer), format, args);
        va_end(args);
        LogInternal(buffer, MessageType::Info);
    }

    void Logger::LogWarning(const char* format, ...) {
        va_list args;
        va_start(args, format);
        char buffer[1024];
        std::vsnprintf(buffer, sizeof(buffer), format, args);
        va_end(args);
        LogInternal(buffer, MessageType::Warning);
    }

    void Logger::LogError(const char* format, ...) {
        va_list args;
        va_start(args, format);
        char buffer[1024];
        std::vsnprintf(buffer, sizeof(buffer), format, args);
        va_end(args);
        LogInternal(buffer, MessageType::Error);
    }

    void Logger::LogDebug(const char* format, ...) {
        va_list args;
        va_start(args, format);
        char buffer[1024];
        std::vsnprintf(buffer, sizeof(buffer), format, args);
        va_end(args);
        LogInternal(buffer, MessageType::Debug);
    }

    void Logger::LogOk(const char* format, ...) {
        va_list args;
        va_start(args, format);
        char buffer[1024];
        std::vsnprintf(buffer, sizeof(buffer), format, args);
        va_end(args);
        LogInternal(buffer, MessageType::Ok);
    }

    const std::vector<Message>& Logger::GetMessages() {
        return m_messages;
    }
}